/*
* Creates a minimum DCC command station 
* 17-05-2018 - v.9
*/
 
#include <DCCPacket.h>
#include <DCCPacketQueue.h>
#include <DCCPacketScheduler.h>

DCCPacketScheduler dcc;

uint16_t Address0 = 89; //adr-2 = 22, out-6
uint16_t Address1 = 84; //adr-1 = 21, out-1
uint16_t Address2 = 85; //adr-1 = 21, out-2
uint16_t Address3 = 93; //adr-1 = 23, out-1
uint16_t Address4 = 94; //adr-1 = 23, out-2
uint16_t Address5 = 97; //adr-2 = 24, out-6

//--------------------------------------------------------------
unsigned long previousT0Millis = 0;    
unsigned long previousT1Millis = 0;

const long intervalT0 = 32;    
const long intervalT1 = 32;    //"0,032 sec."

// FB part:
//--------------------------------------------------------------------------------------------
boolean flag0 = false;    // метка о том, что событие уже посчитали
boolean flag1 = false;
boolean flag2 = false;
boolean flag3 = false;
boolean flag4 = false;
boolean flag5 = false;
boolean flag6 = false;
boolean flag7 = false;

byte M0 = 0;
byte M1 = 0;
byte M2 = 0;

//--------------------------------------------------------------
//R-Bus:
#include <RBus1.h>
RBus1Class RBus1;
//--------------------------------------------------------------
//------ FB-RBus
unsigned int ifb = 0;
bool fbgr0 = false;

//--------------------------------------------------------------


//--------------------------------------------------------------------------------------------
void setup() {
// - - -
  DDRC = 0x00; //pinMode INPUT Reg.C
// - - -
  pinMode(4, OUTPUT);
  digitalWrite(4, LOW);

  pinMode(7, OUTPUT);
  digitalWrite(7, LOW);

  pinMode(13, OUTPUT);
  digitalWrite(13, LOW);
  
  //DCC OUT PIN - DCC Waveform pin
  //INV DCC OUT PIN - inverted DCC Waveform pin
  dcc.setup();  //DCC OUT PIN 9, INV DCC OUT PIN 10

  RBus1.setup(7); //Initialization RBus-Serial1 und Send/Receive-PIN-7

  dcc.setpower(true);
  digitalWrite(4, HIGH); // Enable L293D

}
// END SETUP

//--------------------------------------------------------------------------------------------
void loop() {
  
  unsigned long currentMillis = millis();
// SEND 15E - START FB0
  if(currentMillis - previousT0Millis >= intervalT0) {
    previousT0Millis = currentMillis;
    if (fbgr0 == false) {
     RBus1.XNetsend1(0x15E);
      ifb++;
    }
    else {
      RBus1.FB0(false);
    }
  }
//---
  if (ifb > 500) {
    fbgr0 = true;
    ifb = 0;
  }
//---

  dcc.update();

  currentMillis = millis();
// START FB1
  if(currentMillis - previousT1Millis >= intervalT1) {
    previousT1Millis = currentMillis;
    FB();
  }

  dcc.update();
}

// FB part:
//--------------------------------------------------------------------------------------------
void FB(){

  M0 =~(PINC & B111111); // read pins A0, A1, A2, A3, A4, A5

// датчик 0
  if ((bitRead(M0,0) == true) && (flag0 == false)){    // сравниваем. Если меньше И событие еще не зафиксировано
     dcc.setBasicAccessoryPos(Address1, true);   // светофор прямой красный
     dcc.setBasicAccessoryPos(Address0, false);  // стрелка на боковой      dcc.setBasicAccessoryPos(Address, (Direction & 0x01));
     flag0 = true;                               // ставим метку, что событие зафиксировано
     }
  if ((bitRead(M0,0) == false) && (flag0 == true)){     // опять сравниваем. Если больше И событие уже было зафиксировано
     dcc.setBasicAccessoryPos(Address2, true);   // светофор боковой красный
     dcc.setBasicAccessoryPos(Address0, true);   // стрелка прямо
     flag0 = false;                              // сбрасываем метку, т.е. следующее событие будет снова посчитано
     }

// датчик 1
  if ((bitRead(M0,1) == true) && (flag1 == false)){
     dcc.setBasicAccessoryPos(Address1, false);   // светофор прямой зелёный
     flag1 = true;
     }
  if ((bitRead(M0,1) == false) && (flag1 == true)){
     flag1 = false;
   }
   
// датчик 2
  if ((bitRead(M0,2) == true) && (flag2 == false)){
     dcc.setBasicAccessoryPos(Address2, false);   // светофор боковой зелёный
     flag2 = true;
     }
  if ((bitRead(M0,2) == false) && (flag2 == true)){
     flag2 = false;
     }

// датчик 3 
  if ((bitRead(M0,3) == true) && (flag3 == false)){
     dcc.setBasicAccessoryPos(Address3, false);   // светофор боковой зелёный
     flag3 = true;
     }
  if ((bitRead(M0,3) == false) && (flag3 == true)){
     flag3 = false;
     }

// датчик 4
  if ((bitRead(M0,4) == true) && (flag4 == false)){
     dcc.setBasicAccessoryPos(Address4, false);   // светофор прямой зелёный
     flag4 = true;
     }
  if ((bitRead(M0,4) == false) && (flag4 == true)){
     flag4 = false;
     }

// датчик 5
  if ((bitRead(M0,5) == true) && (flag5 == false)){
     dcc.setBasicAccessoryPos(Address4, true);   // светофор прямой красный
     dcc.setBasicAccessoryPos(Address5, false);  // стрелка на боковой
     flag5 = true;
     }
  if ((bitRead(M0,5) == false) && (flag5 == true)){
     dcc.setBasicAccessoryPos(Address3, true);   // светофор боковой красный
     dcc.setBasicAccessoryPos(Address5, true);   // стрелка прямо
     flag5 = false;
     }
}

//--------------------------------------------------------------------------------------------
//return state of FB sensors
void notifyz21FBData(byte *data) {

  M1 = data[1];
  M2 = data[2];

// датчик 6
  if ((bitRead(M1,4) == true) && (flag6 == false)){ 
    if (dcc.getBasicAccessoryInfo(Address2) == false){
      dcc.setBasicAccessoryPos(Address2, true);   // светофор боковой красный
    }
    if (dcc.getBasicAccessoryInfo(Address1) == false){
     dcc.setBasicAccessoryPos(Address1, true);   // светофор прямой красный
    }
    flag6 = true;
  }
  if ((bitRead(M1,4) == false) && (flag6 == true)){ 
    flag6 = false;
  }

// датчик 7
  if ((bitRead(M2,1) == true) && (flag7 == false)){
    if (dcc.getBasicAccessoryInfo(Address3) == false){
      dcc.setBasicAccessoryPos(Address3, true);   // светофор боковой красный
    }
    if (dcc.getBasicAccessoryInfo(Address4) == false){
      dcc.setBasicAccessoryPos(Address4, true);   // светофор прямой красный
    }
    flag7 = true;
  }
  if ((bitRead(M2,1) == false) && (flag7 == true)){
     flag7 = false;
  }
}

void notifyz21FGR0(bool rfbgr0) {	
	fbgr0 = rfbgr0;
}

//--------------------------------------------------------------------------------------------

